# $Id: Client.pm 29086 2009-10-23 12:03:54Z wsl $
# $URL: https://infix.uvt.nl/its-id/trunk/sources/aselect-perl/lib/Aselect/Client.pm $

use utf8;
use strict;
use warnings FATAL => 'all';;

package Aselect::Client;

use Encode;
use Digest::SHA1 qw(sha1_hex);
use IO::Socket::INET;
use Aselect::Util;

use Exporter 'import';

our @EXPORT = qw(aselect_authenticate aselect_verify_credentials aselect_verify_ticket aselect_attributes);
our @EXPORT_OK = qw(aselect_connect aselect_comm);
our %EXPORT_TAGS = (all => \@EXPORT_OK);  

my $timeout = 2;

sub aselect_read {
	my $fh = shift;
	my $rfds = '';

	vec($rfds, fileno($fh), 1) = 1;
	my $efds = $rfds;

	my $ret = '';

	for(;;) {
		my ($rout, $eout) = ($rfds, $efds);
		my $num = select($rout, undef, $eout, $timeout);
		die "select() failed: $!\n"
			if $num == -1;
		die "timeout waiting for read\n"
			if $num == 0;
		my $buf;
		my $r = sysread($fh, $buf, 65536);
		die "error reading from socket: $!\n"
			unless defined $r;
		last if $r == 0;
		$ret .= $buf;
	}
	return decode_utf8($ret, Encode::FB_CROAK);
}

sub aselect_write {
	my $fh = shift;
	my $msg = shift;
	my $wfds = '';

	vec($wfds, fileno($fh), 1) = 1;
	my $efds = $wfds;

	my $buf = encode_utf8($msg);
	my $len = length($buf);
	my $off = 0;

	for(;;) {
		my ($wout, $eout) = ($wfds, $efds);
		my $num = select(undef, $wout, $eout, $timeout);
		die "select() failed: $!\n"
			if $num == -1;
		die "timeout waiting for write: $!\n"
			if $num == 0;
		my $r = syswrite($fh, $buf, $len - $off, $off);
		die "error writing to socket: $!\n"
			unless defined $r;
		last if $r == 0;
		$off += $r;
		last if $off == $len;
	}
}

sub aselect_connect {
	my $msg = shift;

	my $agent = new IO::Socket::INET(
		Proto => 'tcp',
		PeerAddr => 'localhost',
		PeerPort => 1495,
		Blocking => 0,
		Timeout => $timeout
	);

	die "can't connect to agent: $!\n"
		unless defined $agent;

	binmode($agent)
		or die "can't set socket binmode: $!\n";
#	warn "send: $msg\n";
	aselect_write($agent, $msg);
	$msg = aselect_read($agent);
	warn "recv: $msg\n";

	close($agent)
		or die "can't close socket: $!\n";

	return $msg;
}

sub aselect_comm {
	my $msg = aselect_join(@_) . "\015\012";

	my $reply = aselect_connect($msg);

	my $res = aselect_split($reply);
	my $result_code = delete $res->{result_code};
	die "missing result_code in agent reply\n"
		unless defined $result_code;
	die "result_code=$result_code\n"
		if $result_code ne '0000';
	return $res;
}

sub aselect_authenticate {
	my %args = (request => 'authenticate');
	@args{qw(app_id app_url)} = @_;
	my $res = aselect_comm(%args);
	my $as_url = delete $res->{as_url};
	die "missing as_url in agent reply\n" unless defined $as_url;
	return aselect_url($as_url, $res);
}

sub aselect_verify_credentials {
	my %args = (request => 'verify_credentials');
	@args{qw(rid aselect_credentials)} = @_;
	die "malformed credentials\n" unless aselect_valid(%args);
	my $res = aselect_comm(%args);
	my @fields = qw(uid organization ticket);
	my %res; @res{@fields} = @$res{@fields};
	if(wantarray) {
		my @res = (aselect_join(%res));
		my $attrs = $res->{attributes};
		if(defined $attrs) {
			push @res, aselect_decode($attrs);
		}
		return @res;
	}
	return aselect_join(%res);
}

sub aselect_verify_ticket {
	my %args = (request => 'verify_ticket');
	my $ticket = aselect_split(shift);
	my @fields = qw(uid organization ticket);
	@args{@fields} = @$ticket{@fields};
	$args{ticket_hash} = $ticket->{ticket_hash}
		if defined $ticket->{ticket_hash};
	die "malformed ticket\n" unless aselect_valid(%args);
	my $res = aselect_comm(%args);
	return @args{@fields};
}

sub aselect_attributes {
	my %args = (request => 'attributes');
	my $ticket = aselect_split(shift);
	my @fields = qw(uid organization ticket);
	@args{@fields} = @$ticket{@fields};
	die "malformed ticket\n" unless aselect_valid(%args);
	my $res = aselect_comm(%args);
	$res->{attributes} = aselect_decode($res->{attributes});
	return $res;
}

1;
