# $Id: Database.pm 29027 2009-10-17 12:30:28Z wsl $
# $URL: https://infix.uvt.nl/its-id/trunk/sources/aselect-perl/lib/Aselect/Database.pm $

use utf8;
use strict;
use warnings FATAL => 'all';

package Aselect::Database;

use Aselect::Generator -Base;

field 'cleaned';

sub clean {
	return if $self->cleaned;
	$self->cleaned(1);
	my $cfg = $self->cfg;
	my $timeout = $cfg->timeout;
	return unless $timeout;
	my $db = $self->db;
	my $q = $db->prepare_cached("DELETE FROM requests WHERE ctime < CURRENT_TIMESTAMP - CAST(? AS INTERVAL)");
	$q->execute($timeout);
	$q->finish;
	$q = $db->prepare_cached("DELETE FROM signons WHERE ctime < CURRENT_TIMESTAMP - CAST(? AS INTERVAL)");
	$q->execute($timeout);
	$q->finish;
	# tickets are cascaded by signons
}

sub rid {
	my ($rid, $app_id, $app_url) = @_;

	return undef unless defined $rid;
	return undef unless $rid =~ /^[0-9a-f]+$/i;
	$rid = uc($rid);

	my $db = $self->db;

	$self->clean;

	if(defined $app_id && defined $app_url) {
		my $q = $db->prepare_cached("INSERT INTO requests (rid, app_id, app_url) VALUES (?, ?, ?)");
		$q->execute($rid, $app_id, $app_url);
		$q->finish;
	}

	my $q = $db->prepare_cached("SELECT * FROM requests WHERE rid = ?");
	$q->execute($rid);
	my $res = $q->fetchrow_hashref;
	$q->finish;

	return $res;
}

sub session {
	my ($cookie, $uid, $organization, $attributes) = @_;

	return undef unless defined $cookie;
	return undef unless $cookie =~ /^[0-9a-f]+$/i;
	$cookie = uc($cookie);

	my $db = $self->db;

	$self->clean;

	if(defined $uid && defined $organization) {
		my $q = $db->prepare_cached("INSERT INTO signons (cookie, username, organization, attributes) VALUES (?, ?, ?, ?)");
		$q->execute($cookie, $uid, $organization, $attributes);
		$q->finish;
	}

	my $q = $db->prepare_cached("SELECT * FROM signons WHERE cookie = ?");
	$q->execute($cookie);
	my $res = $q->fetchrow_hashref;
	$q->finish;

	return $res;
}

sub ticket {
	my ($credentials, $session, $info) = @_;

	return undef unless defined $credentials;
	return undef unless $credentials =~ /^[0-9a-f]+$/i;
	$credentials = uc($credentials);

	my $db = $self->db;

	$self->clean;

	if(defined $session && defined $info) {
		my $q = $db->prepare_cached("INSERT INTO tickets (credentials, signon, app_id) VALUES (?, ?, ?)");
		$q->execute($credentials, $session->{signon}, $info->{app_id});
		$q->finish;
	}

	my $q = $db->prepare_cached("SELECT *, EXTRACT(EPOCH FROM tickets.ctime) AS ttime, EXTRACT(EPOCH FROM signons.ctime) AS stime FROM tickets JOIN signons USING (signon) WHERE credentials = ?");
	$q->execute($credentials);
	my $res = $q->fetchrow_hashref;
	$q->finish;

	return $res;
}
